import React, { Fragment } from 'react';

var Step = function Step(id, component, incomingNodes, outgoingNodes) {
  this.id = id;
  this.component = component;
  this.incomingNodes = incomingNodes;
  this.outgoingNodes = outgoingNodes;
};

var Connector = function Connector(component, incomingNodes, outgoingNodes) {
  this.component = component;
  this.incomingNodes = incomingNodes;
  this.outgoingNodes = outgoingNodes;
};

var ListHorizontal = /*#__PURE__*/function () {
  function ListHorizontal(verticalLists, meetNode, color) {
    this.verticalLists = verticalLists;
    this.meetNode = meetNode;
    this.color = color;
  }

  var _proto = ListHorizontal.prototype;

  _proto.addToList = function addToList(verticalList) {
    this.verticalLists = this.verticalLists.concat(verticalList);
  };

  return ListHorizontal;
}();

var ListVertical = function ListVertical(steps, color) {
  this.steps = steps;
  this.color = color;
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var styles = {
  line: {
    borderWidth: 1,
    borderStyle: 'solid'
  }
};

var VerticalLine = function VerticalLine(props) {
  return React.createElement("div", {
    style: _extends({}, styles.line, {
      borderColor: props.borderColor,
      height: props.lineHeight
    })
  });
};

var renderGap = function renderGap(width) {
  return {
    component: React.createElement("div", {
      style: {
        width: width
      }
    })
  };
};

var workflowTreeCreator = function workflowTreeCreator(currentNode, treeData, acc, meetNode, idsCreatedMap, lineColor, lineHeight, gapWidth) {
  if (idsCreatedMap[currentNode.id]) {
    return;
  }

  if (!idsCreatedMap[currentNode.id]) {
    acc.push(new Step(currentNode.id, currentNode.component, currentNode.incomingNodes, currentNode.outgoingNodes));
    acc.push(new Connector(React.createElement(VerticalLine, {
      borderColor: lineColor,
      lineHeight: lineHeight
    }), [currentNode.id], currentNode.outgoingNodes));
    idsCreatedMap[currentNode.id] = true;
  }

  if (currentNode.outgoingNodes[0] === meetNode) {
    return;
  } else if (currentNode.outgoingNodes.length > 1) {
    var localMeetNode = getMeetNodeForListHorizontal(treeData, currentNode.outgoingNodes);
    var currentLength = acc.length;
    var localHorizontalList = new ListHorizontal([], localMeetNode, lineColor);

    if (localMeetNode) {
      workflowTreeCreator(treeData[localMeetNode], treeData, acc, null, idsCreatedMap, lineColor, lineHeight, gapWidth);
      idsCreatedMap[localMeetNode] = true;
    }

    currentNode.outgoingNodes.forEach(function (nodeId) {
      var accumalator = [new Connector(React.createElement(VerticalLine, {
        borderColor: lineColor,
        lineHeight: lineHeight
      }), [currentNode.id], [nodeId])];
      workflowTreeCreator(treeData[nodeId], treeData, accumalator, localMeetNode, idsCreatedMap, lineColor, lineHeight, gapWidth);
      localHorizontalList.addToList(new ListVertical(accumalator, lineColor));
      localHorizontalList.addToList(renderGap(gapWidth));
    });
    acc.splice(currentLength, 0, localHorizontalList);
    acc.splice(currentLength + 1, 0, new Connector(React.createElement(VerticalLine, {
      borderColor: lineColor,
      lineHeight: lineHeight
    }), currentNode.outgoingNodes, []));
  } else if (currentNode.outgoingNodes.length === 1) {
    workflowTreeCreator(treeData[currentNode.outgoingNodes[0]], treeData, acc, meetNode, idsCreatedMap, lineColor, lineHeight, gapWidth);
  } else if (currentNode.outgoingNodes.length === 0) {
    return;
  }
};

var styles$1 = {
  mainList: {
    display: 'flex',
    flexDirection: 'row',
    justifyContent: 'space-between',
    borderBottomWidth: 2,
    borderBottomStyle: 'solid',
    borderTopWidth: 2,
    borderTopStyle: 'solid'
  }
};

var HorizontalList = function HorizontalList(props) {
  var borderColor = props.borderColor,
      steps = props.steps;
  return React.createElement("div", {
    style: _extends({}, styles$1.mainList, {
      borderBottomColor: borderColor,
      borderTopColor: borderColor
    })
  }, steps.map(function (step, index) {
    return React.createElement(Fragment, {
      key: index
    }, step);
  }));
};

var styles$2 = {
  mainList: {
    display: 'flex',
    flexDirection: 'column',
    alignItems: 'center'
  },
  line: {
    flex: 1,
    borderWidth: 1,
    borderStyle: 'solid'
  }
};

var VerticalList = function VerticalList(props) {
  var steps = props.steps,
      borderColor = props.borderColor;
  return React.createElement("div", {
    style: styles$2.mainList
  }, steps.map(function (step, index) {
    return React.createElement(Fragment, {
      key: index
    }, step);
  }), React.createElement("div", {
    style: _extends({}, styles$2.line, {
      borderColor: borderColor
    })
  }));
};

var generateWorkflowTree = function generateWorkflowTree(currentNode) {
  if (currentNode instanceof ListVertical) {
    return React.createElement(VerticalList, {
      steps: currentNode.steps.map(function (node) {
        return generateWorkflowTree(node);
      }),
      borderColor: currentNode.color
    });
  } else if (currentNode instanceof ListHorizontal) {
    return React.createElement(HorizontalList, {
      steps: currentNode.verticalLists.map(function (node) {
        return generateWorkflowTree(node);
      }),
      borderColor: currentNode.color
    });
  } else {
    return currentNode.component;
  }
};
var renderTree = function renderTree(finalList) {
  return generateWorkflowTree(finalList);
};

var orEmptyArray = function orEmptyArray(arr) {
  return arr || [];
};
var getFirstNode = function getFirstNode(treeData) {
  return Object.values(treeData).find(function (node) {
    return node.incomingNodes.length === 0;
  });
};
/**
 * @description This function returns all the possible paths based on the tree data.
 */

var getAllPaths = function getAllPaths(id, treeData, accumlator, finalList) {
  if (treeData[id].outgoingNodes.length === 0) {
    accumlator.push(id);
    finalList.push([].concat(accumlator));
    return;
  }

  accumlator.push(id);
  treeData[id].outgoingNodes.forEach(function (identifier) {
    getAllPaths(identifier, treeData, accumlator, finalList);
    accumlator.pop();
  });
};
/**
 * @description This function returns the smallest length path.
 */

var getSmallestLengthPath = function getSmallestLengthPath(paths) {
  return orEmptyArray(paths.reduce(function (acc, path) {
    return acc && acc.length > path.length ? path : acc;
  }, undefined));
};
/**
 * @description This function returns the meet node.
 */

var getMeetNode = function getMeetNode(paths) {
  var smallestLengthPath = getSmallestLengthPath(paths);
  var meetNode = null;
  smallestLengthPath.every(function (id) {
    var hasAll = true;
    paths.forEach(function (path) {
      if (path.indexOf(id) === -1) {
        hasAll = false;
      }
    });

    if (hasAll === true) {
      meetNode = id;
      return false;
    } else {
      return true;
    }
  });
  return meetNode;
};
/**
 * @description This functions returns the meet node for horizontal list.
 */

var getMeetNodeForListHorizontal = function getMeetNodeForListHorizontal(treeData, outgoingNodes) {
  var acc = [];
  var finalList = [];
  outgoingNodes.forEach(function (node) {
    acc = [];
    getAllPaths(node, treeData, acc, finalList);
  });
  return getMeetNode(finalList);
};
var renderTreeView = function renderTreeView(firstNodeId, treeData, lineColor, lineHeight, gapWidth) {
  if (firstNodeId && treeData) {
    var acc = [];
    workflowTreeCreator(firstNodeId, treeData, acc, null, {}, lineColor, lineHeight, gapWidth);
    return renderTree(new ListVertical(acc, lineColor));
  } else {
    return null;
  }
};

var Tree = function Tree(props) {
  var treeData = props.treeData,
      _props$lineColor = props.lineColor,
      lineColor = _props$lineColor === void 0 ? 'black' : _props$lineColor,
      _props$lineHeight = props.lineHeight,
      lineHeight = _props$lineHeight === void 0 ? 20 : _props$lineHeight,
      _props$gapWidth = props.gapWidth,
      gapWidth = _props$gapWidth === void 0 ? 50 : _props$gapWidth;
  return renderTreeView(getFirstNode(treeData), treeData, lineColor, lineHeight, gapWidth);
};

export { Tree };
//# sourceMappingURL=react-path-tree.esm.js.map
